package com.chelvc.framework.elasticsearch;

import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

import co.elastic.clients.elasticsearch.core.CountResponse;
import co.elastic.clients.elasticsearch.core.GetResponse;
import co.elastic.clients.elasticsearch.core.MgetResponse;
import co.elastic.clients.elasticsearch.core.SearchResponse;
import co.elastic.clients.elasticsearch.core.search.Hit;
import co.elastic.clients.transport.endpoints.BooleanResponse;
import com.chelvc.framework.common.util.ObjectUtils;

/**
 * Elasticsearch工具类
 *
 * @author Woody
 * @date 2024/11/9
 */
public final class ElasticsearchUtils {
    private ElasticsearchUtils() {
    }

    /**
     * 获取数量
     *
     * @param response 响应对象
     * @return 数量
     */
    public static long count(CountResponse response) {
        return response == null ? 0 : response.count();
    }

    /**
     * 获取布尔值
     *
     * @param response 响应对象
     * @return 布尔值
     */
    public static boolean bool(BooleanResponse response) {
        return response != null && response.value();
    }

    /**
     * 获取对象实例
     *
     * @param response 响应对象
     * @param <T>      对象类型
     * @return 对象实例
     */
    public static <T> T source(GetResponse<T> response) {
        return ObjectUtils.ifNull(response, GetResponse::source);
    }

    /**
     * 获取对象查询列表
     *
     * @param response 响应对象
     * @param <T>      对象类型
     * @return 对象列表
     */
    public static <T> List<T> sources(MgetResponse<T> response) {
        if (response == null || ObjectUtils.isEmpty(response.docs())) {
            return Collections.emptyList();
        }
        return response.docs().stream().map(doc -> doc.result().source())
                .filter(Objects::nonNull).collect(Collectors.toList());
    }

    /**
     * 获取对象查询列表
     *
     * @param response 响应对象
     * @param <T>      对象类型
     * @return 对象列表
     */
    public static <T> List<T> sources(SearchResponse<T> response) {
        if (response == null || response.hits() == null || ObjectUtils.isEmpty(response.hits().hits())) {
            return Collections.emptyList();
        }
        return response.hits().hits().stream().map(Hit::source).filter(Objects::nonNull).collect(Collectors.toList());
    }
}
